@@ -20,7 +20,6 @@
 
 import logging
 import traceback
-import collections
 
 from gi.repository import Gtk
 from gi.repository import Gdk
@@ -43,23 +42,22 @@ from .baseclass import vmmGObjectUI
 from .addstorage import vmmAddStorage
 
 (PAGE_ERROR,
-PAGE_DISK,
-PAGE_CONTROLLER,
-PAGE_NETWORK,
-PAGE_INPUT,
-PAGE_GRAPHICS,
-PAGE_SOUND,
-PAGE_HOSTDEV,
-PAGE_CHAR,
-PAGE_VIDEO,
-PAGE_WATCHDOG,
-PAGE_FILESYSTEM,
-PAGE_SMARTCARD,
-PAGE_USBREDIR,
-PAGE_TPM,
-PAGE_RNG,
-PAGE_PANIC,
-) = range(0, 17)
+ PAGE_DISK,
+ PAGE_CONTROLLER,
+ PAGE_NETWORK,
+ PAGE_INPUT,
+ PAGE_GRAPHICS,
+ PAGE_SOUND,
+ PAGE_HOSTDEV,
+ PAGE_CHAR,
+ PAGE_VIDEO,
+ PAGE_WATCHDOG,
+ PAGE_FILESYSTEM,
+ PAGE_SMARTCARD,
+ PAGE_USBREDIR,
+ PAGE_TPM,
+ PAGE_RNG,
+ PAGE_PANIC) = range(0, 17)
 
 
 class vmmAddHardware(vmmGObjectUI):
@@ -93,14 +91,14 @@ class vmmAddHardware(vmmGObjectUI):
         self.addstorage.connect("browse-clicked", self._browse_storage_cb)
 
         self.builder.connect_signals({
-            "on_create_cancel_clicked" : self.close,
-            "on_vmm_create_delete_event" : self.close,
-            "on_create_finish_clicked" : self._finish,
+            "on_create_cancel_clicked": self.close,
+            "on_vmm_create_delete_event": self.close,
+            "on_create_finish_clicked": self._finish,
             "on_hw_list_changed": self._hw_selected,
 
             "on_storage_devtype_changed": self._change_storage_devtype,
 
-            "on_mac_address_clicked" : self._change_macaddr_use,
+            "on_mac_address_clicked": self._change_macaddr_use,
 
             "on_char_device_type_changed": self._change_char_device_type,
             "on_char_target_name_changed": self._change_char_target_name,
@@ -220,7 +218,7 @@ class vmmAddHardware(vmmGObjectUI):
                       "media-cdrom", _("CDROM device")])
         target_model.append([virtinst.VirtualDisk.DEVICE_FLOPPY,
                       "media-floppy", _("Floppy device")])
-        if self.conn.is_qemu() or self.conn.is_test_conn():
+        if self.conn.is_qemu() or self.conn.is_test():
             target_model.append([virtinst.VirtualDisk.DEVICE_LUN,
                           "drive-harddisk", _("LUN Passthrough")])
         target_list.set_active(0)
@@ -317,8 +315,8 @@ class vmmAddHardware(vmmGObjectUI):
         self._build_rng_backend_mode_combo(combo)
 
         # Panic widgets
-        combo = self.widget("panic-type")
-        self._build_panic_address_type(combo)
+        combo = self.widget("panic-model")
+        self._build_panic_models(combo)
 
         # Controller widgets
         combo = self.widget("controller-type")
@@ -405,13 +403,16 @@ class vmmAddHardware(vmmGObjectUI):
                       True, None)
         add_hw_option(_("RNG"), "system-run", PAGE_RNG, True, None)
         add_hw_option(_("Panic Notifier"), "system-run", PAGE_PANIC,
-            self.conn.check_support(self.conn.SUPPORT_CONN_PANIC_DEVICE),
-            _("Not supported for this hypervisor/libvirt combination."))
+            self.conn.check_support(self.conn.SUPPORT_CONN_PANIC_DEVICE) and
+            virtinst.VirtualPanicDevice.get_models(self.vm.get_xmlobj().os),
+            _("Not supported for this hypervisor/libvirt/arch combination."))
 
     def _reset_state(self):
         # Storage init
         self.widget("storage-devtype").set_active(0)
         self.widget("storage-devtype").emit("changed")
+        self.widget("storage-cache").set_active(0)
+        self.widget("disk-advanced-expander").set_expanded(False)
         self.addstorage.reset_state()
 
         # Network init
@@ -466,16 +467,17 @@ class vmmAddHardware(vmmGObjectUI):
             widget.hide()
 
         # RNG params
-        self.widget("rng-device").set_text("/dev/random")
+        default_rng = "/dev/random"
+        if self.conn.check_support(self.conn.SUPPORT_CONN_RNG_URANDOM):
+            default_rng = "/dev/urandom"
+        self.widget("rng-device").set_text(default_rng)
+
         for i in ["rng-bind-host", "rng-connect-host"]:
             self.widget(i).set_text("localhost")
 
         for i in ["rng-bind-service", "rng-connect-service"]:
             self.widget(i).set_text("708")
 
-        # Panic device params
-        self.widget("panic-iobase").set_text("0x505")
-
         # Controller device params
         self._populate_controller_type()
 
@@ -551,7 +553,7 @@ class vmmAddHardware(vmmGObjectUI):
         model.append([None, _("Hypervisor default")])
         if vm.is_hvm():
             mod_list = []
-            if vm.get_hv_type() in ["kvm", "qemu", "test"]:
+            if vm.get_hv_type() in ["kvm", "qemu", "vz", "test"]:
                 mod_list.append("virtio")
             mod_list.append("rtl8139")
             mod_list.append("e1000")
@@ -594,9 +596,8 @@ class vmmAddHardware(vmmGObjectUI):
         vmmAddHardware.populate_smartcard_mode_combo(vm, combo)
 
         idx = -1
-        for rowid in range(len(combo.get_model())):
+        for rowid, row in enumerate(combo.get_model()):
             idx = 0
-            row = combo.get_model()[rowid]
             if row[0] == virtinst.VirtualSmartCardDevice.MODE_DEFAULT:
                 idx = rowid
                 break
@@ -641,9 +642,8 @@ class vmmAddHardware(vmmGObjectUI):
         vmmAddHardware.populate_tpm_type_combo(vm, combo)
 
         idx = -1
-        for rowid in range(len(combo.get_model())):
+        for rowid, row in enumerate(combo.get_model()):
             idx = 0
-            row = combo.get_model()[rowid]
             if row[0] == virtinst.VirtualTPMDevice.TYPE_DEFAULT:
                 idx = rowid
                 break
@@ -716,7 +716,7 @@ class vmmAddHardware(vmmGObjectUI):
                 if "scsi" not in disk_bus_types:
                     disk_bus_types.append("scsi")
 
-            if vm.conn.is_xen() or vm.conn.is_test_conn():
+            if vm.conn.is_xen() or vm.conn.is_test():
                 disk_bus_types.append("xen")
 
         rows = []
@@ -779,7 +779,7 @@ class vmmAddHardware(vmmGObjectUI):
             else:
                 # Guest XML editing
                 define_func(**define_args)
-        except Exception, e:
+        except Exception as e:
             err.show_err((_("Error changing VM configuration: %s") %
                               str(e)))
             return False
@@ -797,7 +797,7 @@ class vmmAddHardware(vmmGObjectUI):
             elif hotplug_args:
                 did_hotplug = True
                 vm.hotplug(**hotplug_args)
-        except Exception, e:
+        except Exception as e:
             did_hotplug = True
             logging.debug("Hotplug failed: %s", str(e))
             hotplug_err = ((str(e), "".join(traceback.format_exc())))
@@ -856,6 +856,8 @@ class vmmAddHardware(vmmGObjectUI):
         _add_row("tablet", "usb")
         _add_row("mouse", "usb")
         _add_row("keyboard", "usb")
+        _add_row("keyboard", "virtio")
+        _add_row("tablet", "virtio")
 
     def _populate_host_device_model(self, devtype, devcap, subtype, subcap):
         devlist = self.widget("host-device")
@@ -886,7 +888,9 @@ class vmmAddHardware(vmmGObjectUI):
         model.clear()
 
         for t in VirtualController.TYPES:
-            if t == VirtualController.TYPE_PCI:
+            if t in [VirtualController.TYPE_IDE,
+                     VirtualController.TYPE_PCI,
+                     VirtualController.TYPE_FDC]:
                 continue
             model.append([t, VirtualController.pretty_type(t)])
 
@@ -973,13 +977,14 @@ class vmmAddHardware(vmmGObjectUI):
         self._build_combo_with_values(combo, types, default)
 
 
-    def _build_panic_address_type(self, combo):
-        types = []
-        for t in virtinst.VirtualPanicDevice.TYPES:
-            types.append([t, virtinst.VirtualPanicDevice.get_pretty_type(t)])
+    def _build_panic_models(self, combo):
+        models = []
+        for m in virtinst.VirtualPanicDevice.get_models(self.vm.get_xmlobj().os):
+            models.append([m, virtinst.VirtualPanicDevice.get_pretty_model(m)])
 
-        self._build_combo_with_values(combo, types,
-                virtinst.VirtualPanicDevice.ADDRESS_TYPE_ISA)
+        self._build_combo_with_values(combo, models,
+                virtinst.VirtualPanicDevice.get_default_model(
+                        self.vm.get_xmlobj().os))
 
 
     #########################
@@ -1153,7 +1158,7 @@ class vmmAddHardware(vmmGObjectUI):
             return
 
         tpm_widget_mappings = {
-            "device_path" : "tpm-device-path",
+            "device_path": "tpm-device-path",
         }
 
         self._dev = VirtualTPMDevice(self.conn.get_backend())
@@ -1196,12 +1201,12 @@ class vmmAddHardware(vmmGObjectUI):
             return
 
         char_widget_mappings = {
-            "source_path" : "char-path",
-            "source_channel" : "char-channel",
-            "source_mode" : "char-mode",
-            "source_host" : "char-host",
-            "bind_host" : "char-bind-host",
-            "protocol"  : "char-use-telnet",
+            "source_path": "char-path",
+            "source_channel": "char-channel",
+            "source_mode": "char-mode",
+            "source_host": "char-host",
+            "bind_host": "char-bind-host",
+            "protocol": "char-use-telnet",
         }
 
         char_class = self._get_char_class()
@@ -1279,7 +1284,7 @@ class vmmAddHardware(vmmGObjectUI):
             try:
                 pool = self.conn.get_pool(poolname)
                 self.idle_add(pool.refresh)
-            except:
+            except Exception:
                 logging.debug("Error looking up pool=%s for refresh after "
                     "storage creation.", poolname, exc_info=True)
 
@@ -1311,7 +1316,7 @@ class vmmAddHardware(vmmGObjectUI):
             if controller is not None:
                 self.vm.attach_device(controller)
             self.vm.attach_device(self._dev)
-        except Exception, e:
+        except Exception as e:
             logging.debug("Device could not be hotplugged: %s", str(e))
             attach_err = (str(e), "".join(traceback.format_exc()))
 
@@ -1335,7 +1340,7 @@ class vmmAddHardware(vmmGObjectUI):
             if controller is not None:
                 self.vm.add_device(controller)
             self.vm.add_device(self._dev)
-        except Exception, e:
+        except Exception as e:
             self.err.show_err(_("Error adding device: %s") % str(e))
             return True
 
@@ -1346,7 +1351,7 @@ class vmmAddHardware(vmmGObjectUI):
         if not error:
             try:
                 failure = self._add_device()
-            except Exception, e:
+            except Exception as e:
                 failure = True
                 error = _("Unable to add device: %s") % str(e)
                 details = "".join(traceback.format_exc())
@@ -1354,9 +1359,7 @@ class vmmAddHardware(vmmGObjectUI):
         if error is not None:
             self.err.show_err(error, details=details)
 
-        self.topwin.set_sensitive(True)
-        self.topwin.get_window().set_cursor(
-            Gdk.Cursor.new(Gdk.CursorType.TOP_LEFT_ARROW))
+        self.reset_finish_cursor()
 
         self._dev = None
         if not failure:
@@ -1366,15 +1369,12 @@ class vmmAddHardware(vmmGObjectUI):
         try:
             if self._validate() is False:
                 return
-        except Exception, e:
+        except Exception as e:
             self.err.show_err(_("Uncaught error validating hardware "
                                 "input: %s") % str(e))
             return
 
-        self.topwin.set_sensitive(False)
-        self.topwin.get_window().set_cursor(
-            Gdk.Cursor.new(Gdk.CursorType.WATCH))
-
+        self.set_finish_cursor()
         progWin = vmmAsyncJob(self._setup_device, [],
                               self._finish_cb, [],
                               _("Creating device"),
@@ -1458,13 +1458,19 @@ class vmmAddHardware(vmmGObjectUI):
                       if x.model == controller_model]
 
         # Save occupied places per controller
-        occupied = collections.defaultdict(int)
+        occupied = {}
         for d in used_disks:
-            if d.get_target_prefix() == disk.get_target_prefix():
+            if (d.get_target_prefix() == disk.get_target_prefix() and
+                d.bus == "scsi"):
                 num = virtinst.VirtualDisk.target_to_num(d.target)
-                occupied[num / 7] += 1
+                idx = num // 7
+                if idx not in occupied:
+                    occupied[idx] = []
+                if d.target not in occupied[idx]:
+                    occupied[idx].append(d.target)
+
         for c in ctrls_scsi:
-            if occupied[c.index] < 7:
+            if c.index not in occupied or len(occupied[c.index]) < 7:
                 controller = c
                 break
         else:
@@ -1483,14 +1489,16 @@ class vmmAddHardware(vmmGObjectUI):
         controller_model = None
         if (bus == "scsi" and
             self.vm.get_hv_type() in ["qemu", "kvm", "test"] and
-            not self.vm.xmlobj.os.is_pseries()):
+            not self.vm.xmlobj.os.is_pseries() and not
+            any([c.type == "scsi"
+                 for c in self.vm.get_controller_devices()])):
             controller_model = "virtio-scsi"
 
         collidelist = [d.path for d in self.vm.get_disk_devices()]
         try:
             disk = self.addstorage.validate_storage(self.vm.get_name(),
                 collidelist=collidelist, device=device)
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(_("Storage parameter error."), e)
 
         if disk is False:
@@ -1513,7 +1521,7 @@ class vmmAddHardware(vmmGObjectUI):
                 disk, controller_model, disks)
 
             disk.generate_target(used, prefer_ctrl)
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(_("Storage parameter error."), e)
 
         if self.addstorage.validate_disk_object(disk) is False:
@@ -1534,10 +1542,6 @@ class vmmAddHardware(vmmGObjectUI):
             return self.err.val_err(_("Network selection error."),
                                     _("A network source must be selected."))
 
-        if not mac:
-            return self.err.val_err(_("Invalid MAC address"),
-                                    _("A MAC address must be entered."))
-
         ret = self._netlist.validate_network(mac, model)
         if ret is False:
             return False
@@ -1573,7 +1577,7 @@ class vmmAddHardware(vmmGObjectUI):
                 raise ValueError(_("invalid listen type"))
             if keymap:
                 self._dev.keymap = keymap
-        except ValueError, e:
+        except ValueError as e:
             self.err.val_err(_("Graphics device parameter error"), e)
 
     def _validate_page_sound(self):
@@ -1582,7 +1586,7 @@ class vmmAddHardware(vmmGObjectUI):
         try:
             self._dev = virtinst.VirtualAudio(self.conn.get_backend())
             self._dev.model = smodel
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(_("Sound device parameter error"), e)
 
     def _validate_page_hostdev(self):
@@ -1608,7 +1612,7 @@ class vmmAddHardware(vmmGObjectUI):
                     return False
             dev.set_from_nodedev(nodedev)
             self._dev = dev
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(_("Host device parameter error"), e)
 
     def _validate_page_char(self):
@@ -1652,11 +1656,11 @@ class vmmAddHardware(vmmGObjectUI):
             source_host = source_port = source_mode = None
 
         value_mappings = {
-            "source_path" : source_path,
-            "source_channel" : source_channel,
-            "source_mode" : source_mode,
-            "source_host" : source_host,
-            "source_port" : source_port,
+            "source_path": source_path,
+            "source_channel": source_channel,
+            "source_mode": source_mode,
+            "source_host": source_host,
+            "source_port": source_port,
             "bind_port": bind_port,
             "bind_host": bind_host,
             "protocol": protocol,
@@ -1673,7 +1677,7 @@ class vmmAddHardware(vmmGObjectUI):
 
             # Dump XML for sanity checking
             self._dev.get_xml_config()
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(
                     _("%s device parameter error") %
                     char_class.virtual_device_type.capitalize(), e)
@@ -1685,7 +1689,7 @@ class vmmAddHardware(vmmGObjectUI):
         try:
             self._dev = VirtualVideoDevice(conn)
             self._dev.model = model
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(_("Video device parameter error"), e)
 
     def _validate_page_watchdog(self):
@@ -1697,7 +1701,7 @@ class vmmAddHardware(vmmGObjectUI):
             self._dev = VirtualWatchdog(conn)
             self._dev.model = model
             self._dev.action = action
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(_("Watchdog parameter error"), e)
 
     def _validate_page_filesystem(self):
@@ -1712,7 +1716,7 @@ class vmmAddHardware(vmmGObjectUI):
         try:
             self._dev = VirtualSmartCardDevice(conn)
             self._dev.mode = mode
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(_("Smartcard device parameter error"), e)
 
     def _validate_page_usbredir(self):
@@ -1731,7 +1735,7 @@ class vmmAddHardware(vmmGObjectUI):
                 self._dev.host = host
             if service:
                 self._dev.service = service
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(_("USB redirected device parameter error"),
                                     str(e))
 
@@ -1742,7 +1746,7 @@ class vmmAddHardware(vmmGObjectUI):
         device_path = self.widget("tpm-device-path").get_text()
 
         value_mappings = {
-            "device_path" : device_path,
+            "device_path": device_path,
         }
 
         try:
@@ -1751,25 +1755,18 @@ class vmmAddHardware(vmmGObjectUI):
             for param_name, val in value_mappings.items():
                 if self._dev.supports_property(param_name):
                     setattr(self._dev, param_name, val)
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(_("TPM device parameter error"), e)
 
     def _validate_page_panic(self):
         conn = self.conn.get_backend()
 
-        iobase = self.widget("panic-iobase").get_text()
-
-        value_mappings = {
-            "iobase" : iobase,
-        }
+        model = uiutil.get_list_selection(self.widget("panic-model"))
 
         try:
             self._dev = VirtualPanicDevice(conn)
-            if not iobase:
-                iobase = self._dev.IOBASE_DEFAULT
-            for param_name, val in value_mappings.items():
-                setattr(self._dev, param_name, val)
-        except Exception, e:
+            self._dev.model = model
+        except Exception as e:
             return self.err.val_err(_("Panic device parameter error"), e)
 
     def _validate_page_controller(self):
@@ -1847,13 +1844,13 @@ class vmmAddHardware(vmmGObjectUI):
                                      _("The EGD service must be specified."))
 
         value_mappings = {
-            "backend_type" : backend_type,
-            "backend_source_mode" : backend_mode,
-            "connect_host" : connect_host,
-            "connect_service" : connect_service,
-            "bind_host" : bind_host,
-            "bind_service" : bind_service,
-            "device" : device,
+            "backend_type": backend_type,
+            "backend_source_mode": backend_mode,
+            "connect_host": connect_host,
+            "connect_service": connect_service,
+            "bind_host": bind_host,
+            "bind_service": bind_service,
+            "device": device,
         }
 
         try:
@@ -1862,7 +1859,7 @@ class vmmAddHardware(vmmGObjectUI):
             for param_name, val in value_mappings.items():
                 if self._dev.supports_property(param_name):
                     setattr(self._dev, param_name, val)
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(_("RNG device parameter error"), e)
 
 
