@@ -47,14 +47,14 @@ class vmmMigrateDialog(vmmGObjectUI):
         self._conns = {}
 
         self.builder.connect_signals({
-            "on_vmm_migrate_delete_event" : self._delete_event,
-            "on_migrate_cancel_clicked" : self._cancel_clicked,
-            "on_migrate_finish_clicked" : self._finish_clicked,
+            "on_vmm_migrate_delete_event": self._delete_event,
+            "on_migrate_cancel_clicked": self._cancel_clicked,
+            "on_migrate_finish_clicked": self._finish_clicked,
 
-            "on_migrate_dest_changed" : self._destconn_changed,
-            "on_migrate_set_address_toggled" : self._set_address_toggled,
-            "on_migrate_set_port_toggled" : self._set_port_toggled,
-            "on_migrate_mode_changed" : self._mode_changed,
+            "on_migrate_dest_changed": self._destconn_changed,
+            "on_migrate_set_address_toggled": self._set_address_toggled,
+            "on_migrate_set_port_toggled": self._set_port_toggled,
+            "on_migrate_mode_changed": self._mode_changed,
         })
         self.bind_escape_key_close()
 
@@ -108,13 +108,16 @@ class vmmMigrateDialog(vmmGObjectUI):
         model.set_sort_column_id(COL_LABEL, Gtk.SortType.ASCENDING)
 
         def _sorter(model, iter1, iter2, ignore):
+            def _cmp(a, b):
+                return ((a > b) - (a < b))
+
             row1 = model[iter1]
             row2 = model[iter2]
             if row1[COL_URI] is None:
                 return -1
             if row2[COL_URI] is None:
                 return 1
-            return cmp(row1[COL_LABEL], row2[COL_LABEL])
+            return _cmp(row1[COL_LABEL], row2[COL_LABEL])
         model.set_sort_func(COL_LABEL, _sorter)
 
         # Mode combo
@@ -352,9 +355,7 @@ class vmmMigrateDialog(vmmGObjectUI):
         return uri
 
     def _finish_cb(self, error, details, destconn):
-        self.topwin.set_sensitive(True)
-        self.topwin.get_window().set_cursor(
-            Gdk.Cursor.new(Gdk.CursorType.TOP_LEFT_ARROW))
+        self.reset_finish_cursor()
 
         if error:
             error = _("Unable to migrate guest: %s") % error
@@ -378,16 +379,14 @@ class vmmMigrateDialog(vmmGObjectUI):
                 uri = self.widget("migrate-tunnel-uri").get_text()
             else:
                 uri = self._build_regular_migrate_uri()
-        except Exception, e:
+        except Exception as e:
             details = "".join(traceback.format_exc())
             self.err.show_err((_("Uncaught error validating input: %s") %
                                str(e)),
                                details=details)
             return
 
-        self.topwin.set_sensitive(False)
-        self.topwin.get_window().set_cursor(
-            Gdk.Cursor.new(Gdk.CursorType.WATCH))
+        self.set_finish_cursor()
 
         cancel_cb = None
         if self.vm.getjobinfo_supported:
@@ -413,7 +412,7 @@ class vmmMigrateDialog(vmmGObjectUI):
 
         try:
             vm.abort_job()
-        except Exception, e:
+        except Exception as e:
             logging.exception("Error cancelling migrate job")
             asyncjob.show_warning(_("Error cancelling migrate job: %s") % e)
             return
